#if !defined(FUNC_H)
#define FUNC_H

#include "object.h"

template <class FunctionType>
class Function: public Object {

/*
    A class of user-defined functions.  There are three types of 
    user-defined functions:

      . TestFunction
      . ComputedConceptFunction
      . ComputedFillerFunction

    Therefore this class is defined as a template class.
 */

public:

   ~Function()
    {
	assert(OB.erase(_name));
    }

    Type type() const { return function; }

    String type_name() const { return "function"; }

    String name() const { return _name; }

    FunctionType func() const { return _func; }

    void print(ostream& out = cout) const
    {
	Object::print(out); out << "Name: " << _name << endl; 
    }

    void pretty_print(ostream& out = cout) const 
    {
	out << _name << "()"; 
    }

public:

    /*
	As in define_concept and define_individual... the ctor for
	Function is protected because one must use 'new' to define
	a function.  The reason for this is the created pointer is
	registered into the ObjectBase and is 'deleted' automatically
	in ~ObjectBase.

	HOWEVER, I can't put the following two template functions 
        (define_function and define_test) inside this class and
	declare them as friends.  

	(define_test is obsolete)
     */

    Function(String name, FunctionType func): 
	Object(name), _name(name), _func(func) {}

private:

    String _name;

    FunctionType _func;
};


#if 0

/*
    Why can't I do the following?
 */

    template <class Type>
    inline Function<Type>* define_function(const String& name, Type func) 
    { 
	 return new Function<Type>(name, func); 
    }

/*
    For a further test, I have tried this.  Still don't understand why
    the following failed to compile:

    #include <iostream.h>
    #include <String.h>

    template <class T>
    inline int f(String x, T y) // will succeed if String is replaced by char*
    {
	return 0;
    }

    main()
    {
	f("test", 1);
	return 0;
    }

 */

/*
    As a followup, I did find the above is prohibited checking ARM. 
    The reason is there will be a new function generated for every
    type, not just matching.  So, the load may be bigger than the 
    programmer's expectation.  I guess doesn't hurt AI researchers.
 */
#else

    template <class FuncType>
    inline Function<FuncType>* define_function(
	const char* name, 
	FuncType func
    ) { return new Function<FuncType>(name, func); }

#endif


#endif
