#if !defined(OBJECT_H)
#define OBJECT_H

#include "object_base.h"

class Object {

/*
    Just a general object class.  Used as the root of all objects in
    the system.  

    Objects have types given by the user and unique ids given by the 
    system when an object is registered by the insert() methods.

    The older design has named and unnamed objects.  It becomes clear
    that objects only need to have ids.  The reason is 1. there can
    be unnamed objects any way, so name seems to be the property of
    descendants of Object;  2. there are some subclasses of Object 
    that allow multiple names for an object, e.g., Concept and 
    PrimitiveConcept.  So, we might as well make name a property of
    lower levels since the definition of name might vary.
 */

public:

    enum Type {
        object,
        concept, 
        primitive_concept, 
        disjoint_primitive_concept, 
        individual, 
        role, 
        rule,
        role_restriction,
	procedure,
	test,
	computed_concept,
	computed_fillers,
	function
    };

    /*
        Construct an unnamed/named object.
     */

    Object();

    Object(const String& name);

    virtual ~Object();

    Id id() const { return _id; }

    virtual Type type() const { return object; }

    virtual String type_name() const { return "object"; }

    virtual String name() const { return "Unknown"; }

    virtual void print(ostream& out=cout) const;

    virtual void pretty_print(ostream& out=cout) const;

    friend ostream& operator<<(ostream& out, const Object& x)
    {
	x.pretty_print(out);
	return out;
    }

    static ObjectBase& object_base;

    /*
        The default action of each is error because only HostIndividual
	can be casted into host values.
     */ 

    virtual operator int() const    { assert(0); }

    virtual operator float() const  { assert(0); }

    virtual operator String() const { assert(0); }

private:

    Id _id;
};

extern Object* object(const String& name);

#endif
