#if !defined(OBJECT_BASE_H)
#define OBJECT_BASE_H

#include "typedef.h"

class ObjectBase {

/*
    The object-base consists of two static members in class Object.

        world        - an unordered vector of polymorphic objects pointers, 
                       the index of each serves as a unique id.

		       It is assumed that each of the objects in the world
		       is created by the new operator.  Thus, delete will
		       be called on each object in ObjectBase's dtor.

        world_map    - a map mapping object name to its integer id.  Unnamed 
                       objects do not appear in here.  They can be accessed
                       only through their ids.
 */

public:

    /*
        Initialize the whole knowledge base.
     */
    ObjectBase();

    /*
        Call "delete" operator on each element of the world vector.
        The dtor of Object will then call back the erase method to
        set 0 into the vector element.
     */
   ~ObjectBase();

    /*
        Insertion of objects
     */
    Id insert(Object* object);

    /*
        Insert <name,id> pair into the world map
     */
    bool insert(const String& name, Id id);

    /*
        Erase id from the world
     */
    bool erase(Id id);

    /*
        Erase name from the world map.
     */
    bool erase(const String& name);

    /*
        Accessors to the world map
     */
    Object* find(const String& name);

    bool exists(const String& name);

    bool initialized() const { return _init; }

    unsigned size() const { return world.size(); }

    /*
        Some built-in objects (initialized by the constructor)
     */
    Concept* thing;
    Concept* classic_thing;
    Concept* host_thing;
    Concept* string_thing;
    Concept* number_thing;
    Concept* integer;
    Concept* floating;

public:  // should be private after debug ###

    ObjectVector world;
    Map<String, Id> world_map;
    bool _init;

    /*
        Some define_ functions are allowed access to the above data
        structures.
     */
};

extern ObjectBase OB;

#endif
