#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#iclude "individual.h"
#include "concept.h"
#include "user.h"


char* add_space(char* cmd, char* s)
/*
   This function is used to make sure the command "cmd" has
   proper spacing.  The following rules are used:

   . i/o redirections < and > are separated by whitespace

 */
{
    char *start = s;

    if (!cmd) {
        *s = 0;
        return 0;
    } 

    *s++ = *cmd++;   /* assume the first is not < or >, since need to BT */
   
    for ( ; *cmd; ) {
        if (*cmd == '<' || *cmd == '>') {
	    if (!isspace(*(cmd-1))) 	/* no space in front? */
		*s++ = ' ';
	    *s++ = *cmd++;
	    if (!isspace(*cmd))		/* no space after? */
                *s++ = ' ';
	} else
	    *s++ = *cmd++;
    }	

    *s = 0;

    return start;
}


/*
   Parsing a command line (assumed already history/alias/variable
   expanded by UNIX) into name/options/optargs/arguments is not as
   simple as it may seem at first glance.  Consider the following:

      ls -la
      lpr -P5a
      enscript -2rGP5a main.c
      gcc -ansi main.c
      gcc -c main.c
      gcc -o main main.c

   The categories of options:

    . letter options.  		-la is the same as -l -a
    . word options.  		-ansi is not -a -n -s -i
    . augmented options. 	-P5a and -o main (some require space)
    . combo.  			-2rGP5a is actually -2 -r -G -P5a

   We need to consult the encoded syntactic knowledge to do parsing.
   The complication is that many utilities do not follow the same 
   parsing methodology for options parsing, as a result, it is 
   truly impossible for one parsing program to parse all command
   lines.  We choose to use the "getopt" methodology because it is
   a standard built-in parsing method.

   However, we even foresee the getopt method will be phased out 
   soon because it does not parse word options, which we think would
   become the standard due to its lesser memory load and friendliness.

   Currently the utilities that use getopt(3) are:

   /usr/ucb/	/bin/
      mail	  date
      from	  rcp
      ul	  
      vacation

   There are certainly many others that do not use getopt explicitly
   but follow the same getopt rules.

   Return the individual if OK.
	  NULL if empty.
	  -1 if C-CLASSIC error.
*/
Individual* test_parse(char* cmd)
{
    int n, parents_n;
    char cmd2[1024];
    char *a[100], *s=cmd2;
    int command_found, i_done, o_done;

    // ------ Partition the command line into words -------------------

    add_space(cmd, cmd2);
    n = 0;
    while(1) {
	while (isspace(*s)) 
	    s++;
        if (!*s) {
	    a[n] = 0;
	    break;	
	}
	if (*s == '\'') {
	    a[n++] = ++s;
	    while (*s && *s != '\'')
		s++;
	} else
	if (*s == '"') {
	    a[n++] = ++s;
	    while (*s && *s != '"')
		s++;
	} else {
	    a[n++] = s;
            while (*s && !isspace(*s))
		s++;
	}
	if (!*s) {
	    a[n] = 0;
	    break;
	}
	*s++ = 0;
    }

    if (!a[0])
	return NULL;

    // ----------------------------------------------------------------

    char temp[2048];
    char **line = a;
    char *name = *line++;	/* Name found, try to create an individual */
    cout << "The command name is " << name << ".  ";

    Individual* ind = define_individual(newname("COMMAND"), and(
	concept("COMMAND"),
	fills(role("name"), ind(name), 0),
	0));
    if (!ind) {
        cerr << "Cannot define individual.  parse.C (" << __LINE__ << ")\n";
	return (Individual*)(-1);
    }
    cout << "An individual " << ind->name() << " is created.\n";

    /* check the parents.  it should not contain COMMAND unless the 
       command name is spelled wrong */
    const Set<Concept*> parents = ind->parents();
    if (parents.empty()) {
        cerr << "Individual has no parents.  parse.C (" << __LINE__ << ")\n";
	return (Individual*)(-1);
    }
    parents_n = parents.size();
    if (parents.find(concept("COMMAND") != parents.end()) {
	cout << "COMMAND is a direct parent of " << ind->name() <<
	    ", meaning no concept is encoded\n" <<
	    "(besides COMMAND) that describes " << name << ".\n";
	command_found = 0;
    } else {
	command_found = 1;
    }

    /*
	Here, we need to get the optstr from the command concept, in order
	to proceed the parsing.  This makes the shell more knowledgeable 
	than an ordinary UNIX shell.  In essence, an ordinary shell parses
	options and optargs without any prior knowledge, for example:

	    mail -s subject so < input

	"subject" will be just an argument.  In the ai shell, subject will
	be a subject because the shell "knows" -s is the option for subject
	line.
     */

    String optstr;
    if (*line && **line == '-') {
	cout << "Parsing options\n";
	Individual* opt = ind->filler("optstr");
	if (opt)
	    optstr = String(*opt);
	else
	    optstr = "";
    }

    String fills_options, fills_optargs;

    while (*line && **line == '-') {
	char *c, *where;
        for (c = *line++ + 1; *c; c++) { /* for every letter in this option */
            where = strchr(optstr, *c);
/*
	    if (!where && command_found)
		cout << "    -" << *c << "is not encoded\n";
 */
	    
	    if (where && where[1] == ':') {   /* augmented option */
		if (c[1])   /* no space */
		    where = &c[1];
		else        /* next argument is needed */
		    if (!line) {
			cout << "    -" << *c << "lacks an argument\n";
			continue;
		    } else
			where = *line++;

		fills_options = fills_options + '-' + *c + ' ';
                fills_optargs = fills_optargs + '-' + *c + ' ' + where + ' ';

		cout <<  "    -" << *c << " " << where << endl;
		c = strchr(c, 0) - 1;
	    } else {                 /* single letter option */
		fills_options = fills_option + ' ' + *c + ' ';
/*		if (where) */
		    cout << "    -" << *c << endl;
	    }
        }
    }

    String expr;

    if (fills_options != "") 
	expr = expr + "(fills options " + fills_options + ") ";

    if (fills_optargs != "")
	expr = expr + "(fills optargs " + fills_optargs + ") ";

    if (*line)
	cout << "\nParsing arguments\n";

    String args;
    for ( ; *line && !(**line == '<' || **line == '>'); line++) {
	Individual* id;
	char temp[256];
	id = define_individual(newname("IDENTIFIER"), 
	    and(concept("IDENTIFIER"),
		fills(role("name"), ind(*line), 0),
	    0));
	if (!id) {
	    cerr << "Can't create IDENTIFIER.  parse.C (" << __LINE__ << ")\n";
	    return (Individual*)(-1);
	}
	args = args + id->name() + " ";
	cout << "    " << *line << "(" << id->name() << ") isa ";
	sprintf(temp, "(ind-parents %s)", cl_ind_name(id));
	evaluate("(ind-parents " + id->name() + ")");
    }
    if (expr[0]) {
	strcat(expr, ")");
	cl_expr_eval(expr);
    }

    cl_ind_close_role(ind, cl_object_value("options"));
    cl_ind_close_role(ind, cl_object_value("optargs"));
    cl_ind_close_role(ind, cl_object_value("arguments"));

/*
    Check for I/O redirection.

    Look for < and >.  The argument following each is considered to
    be the file name for redirection.
 */
    for (i_done = o_done = 0; *line; line++) {
	cl_object id;

        if (**line == '<') {
 	    printf("\tinput: ");
	    line++;
	    if (!*line)
		printf("none??\n");
	    else {
		printf("%s\n", *line);
		id = cl_define_ind(newname("IDENTIFIER"), 
		    cl_and(
			cl_object_value("IDENTIFIER"),
			cl_fills( cl_object_value("name"), 
			cl_string_object(*line), 0),
		    0));
		if (!id) {
		    cl_print_cur_error(__LINE__, 1, 0);
		    return (cl_object)(-1);
		}
		cl_ind_add_filler(ind, cl_object_value("input"), id);
		i_done = 1;
	    }
	} else
	if (**line == '>') {
 	    printf("\toutput: ");
	    line++;
	    if (!*line)
		printf("none??\n");
	    else {
		printf("%s\n", *line);
		id = cl_define_ind(newname("IDENTIFIER"), 
		    cl_and(
			cl_object_value("IDENTIFIER"),
			cl_fills( cl_object_value("name"), 
			cl_string_object(*line), 0),
		    0));
		if (!id) {
		    cl_print_cur_error(__LINE__, 1, 0);
		    return (cl_object)(-1);
		}
		cl_ind_add_filler(ind, cl_object_value("output"), id);
		o_done = 1;
	    }
	}
    }

    if (!i_done) {
	printf("\tinput: keyboard\n");
	cl_ind_add_filler(ind, cl_object_value("input"), 
	    cl_object_value("keyboard"));
    }

    if (!o_done) {
	printf("\toutput: screen\n");
	cl_ind_add_filler(ind, cl_object_value("output"), 
	    cl_object_value("screen"));
    }

    if (command_found) {
	parents = cl_ind_parents(ind);
	if (!parents) {
	    cl_print_cur_error(__LINE__, 1, 0);
	    return (cl_object)(-1);
	}
	parents_n = cl_vector_size(parents);
	printf("\n%smmediate parent%s of %s %s:\n    ", 
	    parents_n == 1? "The i": "I",
	    parents_n > 1? "s": "",
	    cl_ind_name(ind),
	    parents_n > 1? "are": "is");
        cl_map_print(parents, cl_concept_name);
	printf("\n");
    }

    printf("\n");

    return ind;
}

