#include "trace.h"
#include "procedure.h"
#include "individual.h"


void ProceduralAttachment::print(ostream& cout) const
{
    Object::print(cout);

    cout << "Name: " << name() << endl;

    cout << "Arguments: " << endl;
    for (int i = 0; i < _argument.size(); i++) {
	cout << "    [" << i << "] ";
        cout << _argument[i]->name();
	cout << endl;
    }
}


void ProceduralAttachment::pretty_print(ostream& out) const
{
    out << _name << "(";
    for (int i = 0; i < _argument.size(); i++) {
	if (_argument[i]->type() == Object::individual &&
	    ((Individual*)_argument[i])->host())
	    cout << *_argument[i];
	else
	    out << _argument[i]->name();
	if (i < _argument.size()-1)
	    out << ", ";
    }
    out << ")";
}


TestResult Test::execute(Individual* I)
{
    if (trace.on(Trace::test))
	cout << "Testing " << *this << endl;

    TestResult result = _func((ClassicIndividual*)I, argument());

    if (I->_test_results.find(this) != I->_test_results.end())
	I->_test_results.erase(this);

    I->_test_results.insert(pair<Test* const, TestResult>(this, result));

    if (trace.on(Trace::test)) {
	cout << "Test returns ";
	switch (result) {
	    case MAYBE: cout << "maybe\n"; break;
	    case YES: cout << "yes\n"; break;
	    case NO: cout << "no\n"; break;
	}
    }

    return result;
}


Concept* ComputedConcept::execute(Individual* I) const
{
    if (trace.on(Trace::rule))     
	cout << "Firing CC " << *this << endl;

    Concept* C = _func((ClassicIndividual*)I, argument());

    if (trace.on(Trace::rule))
	cout << "CC returns " << *C << endl;

    return C;
}


ObjectVector ComputedFillers::execute(Individual* I) const
{
    if (trace.on(Trace::rule))
	cout << "Firing CF " << *this << endl;

    ObjectVector vec = _func((ClassicIndividual*)I, role(), argument());

    if (trace.on(Trace::rule)) {
	cout << "CF returns ";
	for (int i = 0; i < vec.size(); i++)
	    cout << *vec[i] << " ";
	cout << endl;
    }

    return vec;
}
