#if !defined(ROLE_H)
#define ROLE_H

#include "object.h"

class Role: public Object {

/*
    A role is simply a symbolic name of any "has-a" relationship.
    A role is also registered into the OB.  
 */

    friend Role* define_role(const String& name);
    friend Role* define_attribute(const String& name);

protected:

    /*
	Just like Concept and Individual, a role must be declared by
	calling define_role() (or define_attribute).
     */

    Role(const String& name, bool attr = FALSE): 
	Object(name), _name(name), _attribute(attr) {}

public:

   ~Role() { assert(OB.erase(_name)); }
    
    bool attribute() const { return _attribute; }

    String name() const { return _name; }

    Type type() const { return role; }

    String type_name() const { return "role"; }

    void print(ostream& out = cout) const;

    void pretty_print(ostream& out = cout) const 
    {
        out << *this;
    }

    friend ostream& operator<<(ostream& out, const Role& R) 
    {
	out << R._name;
	return out;
    }

private:
  
    bool _attribute;
    String _name;
};

extern Role* role(const String& name);

#endif
