#if !defined(ROLE_RESTR_H)
#define ROLE_RESTR_H

#include "concept.h"
#include "role.h"

class RoleRestriction: public Object {
    
/*
    A RoleRestriction is a description of the potential fillers of a 
    role.  It is also a registered Object. 
 */

    friend Concept* all(Role*, Concept*);
    friend Concept* at_least(int, Role*);
    friend Concept* at_most(int, Role*);
    friend Concept* exactly(int, Role*);
    friend Concept* between(int, int, Role*);
    friend Concept* fills(Role*, const List<Individual*>&);
    friend void merge_role_restriction(RoleRestriction*, 
		const RoleRestriction*);
    friend class ClassicIndividual;

protected:
 
    RoleRestriction(Role* R): _role(R), _vr(Thing), _closed(FALSE)
    {
	if (R->attribute())
	    _nr = pair<int,int>(1,1);
	else
	    _nr = pair<int,int>(-1,-1);
    }

    void add_filler(Individual*);

public:

    RoleRestriction(const RoleRestriction&);

   ~RoleRestriction() {}

    Type type() const { return role_restriction; }

    String type_name() const { return "role restriction"; }

    String name() const { return role()->name(); }

    const Role* role() const { return _role; }

    pair<int, int> nr() const { return _nr; }

    int low() const { return _nr.first; }

    int high() const { return _nr.second; }

    const Concept* vr() const { return _vr; }

    const List<Individual*>& fillers() const { return _fillers; }

    unsigned number() const { return _fillers.size(); }

    bool contradict() const;

    friend ostream& operator<<(ostream&, const RoleRestriction&);

    void print(ostream& out = cout) const 
    {
	Object::print(out); out << *this << endl; 
    }

    void pretty_print(ostream& out = cout) const
    {
	out << *this;
    }

    friend bool operator==(const RoleRestriction&, const RoleRestriction&);

    friend bool operator<=(const RoleRestriction&, const RoleRestriction&);

    friend bool operator<(const RoleRestriction&, const RoleRestriction&);

    friend bool operator>(const RoleRestriction&, const RoleRestriction&);

    friend bool operator>=(const RoleRestriction&, const RoleRestriction&);

    void close();

    bool closed() const { return _closed; }

private:

    bool contradict_nr() const;

    bool contradict_vr() const;

    bool contradict_fillers() const;

private:

    Role* _role;

    /*
	It is very important that the vr must be a real object (i.e.,
	must be new-ed).  This is because for such a concept:
	 
	   and( all(R,C1), all(R,C2), 0 )

	C2 will be merged into C1 during normalization.  If only a
	pointer is kept, the original C1 will be altered.  The same
        thing is true when a Concept itself makes a copy of a
	RoleRestriction, e.g.,

           Concept* C = all(R,C1);
	   define_concept( "X", C );    // must make private copy
	   define_concept( "Y", C );    // otherwise this points to the
					// same RR 
     */

    Concept* _vr;	     // value restriction

    pair<int, int> _nr;	     // number restriction .. i.e, low..high #fillers

    List<Individual*> _fillers;

    bool _closed;
};

#endif
