#if !defined(TRACE_H)
#define TRACE_H

#include <bool.h>


/*
    The trace command is better implemented by trace levels instead of
    on/off switch.  Currently, the trace levels are:

	rule 		-- trace the firing of rules
	test		-- trace tests
	classification	-- trace the top level taxonomic classification.
	subsumption	-- trace the complete subsumption comparison mechanism.
 */

class Trace {
    
public:

    typedef enum { 
	rule, test, classification, subsumption
	// ........................................  
    } Level;                //                    :   remember to adjust
                            //                    :   the size.
    #define Level_n 4       // ...................:

    Trace() 
    {
	for (int i = 0; i < Level_n; i++) 
	    _status[i] = FALSE;
	_suspended = FALSE;
    }

    void set(Level i);

    void unset(Level i);

    void suspend();

    void resume();

    bool on(Level) const;

    bool off(Level i) const 
    {
	return !on(i);
    }

private:

    bool _status[Level_n];

    bool _suspended;
};

extern Trace trace;

#endif
