#if !defined(TYPEDEF_H)
#define TYPEDEF_H

#include <String.h>
#include <stl_.h>
#include <stdarg.h>
#include <iostream.h>
#include <assert.h>
#include <bool.h>

class Object;
class Concept;
class PrimitiveConcept;
class DisjointPrimitiveConcept;
class Individual;
class ClassicIndividual;
class Role;
class RoleRestriction;
class Rule;
class SimpleRule;
class ConceptRule;
class Function;
class ProceduralAttachment;
class Test;
class ComputedConcept;

typedef int Id;


/*
    Return values for test functions.
 */

enum TestResult {
    YES, NO, MAYBE
};

/*
    Declare a vector of objects... a frequently used data structure.
 */

typedef Vector<Object*> ObjectVector;


/*
    TestFunction is a type for test functions.  A test function has
    the following prototype:

	TestResult func(const Individual*, const ObjectVector& args);

    Note: new definition omits "int arg_n" because ObjectVector has a
	  size() parameter.

	  newer definition makes a test accepts ClassicIndividual.  I
	  don't see too much use for test_h.  If the need arises, we 
	  can still have define another TestFunction that takes 
	  HostIndividual as the first param.
 */

typedef TestResult (*TestFunction)(const ClassicIndividual*, const ObjectVector&);

inline ostream& operator<<(ostream& out, TestResult x)
{
    switch (x) {
    case YES: 
	out << "Yes"; break;
    case NO: 
	out << "No"; break;
    case MAYBE: 
	out << "Maybe"; break;
    }
    return out;
}

/*
    ComputedConceptFunction is a type for the C functions that are
    called by computed concepts rules.  A ComputedConceptFunction has
    the following prototype:

	Concept* func(const ClassicIndividual*, const ObjectVector& args);
 */

typedef Concept* (*ComputedConceptFunction)(
                     const ClassicIndividual*, 
		     const ObjectVector&
		 );


typedef ObjectVector (*ComputedFillersFunction)(
                     const ClassicIndividual*, 
		     const Role*,
		     const ObjectVector&
		 );

#endif
